@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: registry.py 1359 2012-02-27 17:11:21Z schindlerf $
+# $Id: registry.py 2123 2012-11-20 15:22:17Z schindlerf $
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -37,7 +37,6 @@ modularity, extensibility and flexibility of EOxServer.
 
 import imp
 import os.path
-import logging
 from inspect import isclass
 
 from django.conf import settings
@@ -153,7 +152,22 @@ class Registry(object):
 
     def findImplementations(self, intf_id, params=None, include_disabled=False):
         """
+        This method returns a list of implementations of a given interface.
+        It requires the interface ID as a parameter.
         
+        Furthermore, a parameter dictionary can be passed to the method. The
+        results will then be filtered according to these parameters. The
+        dictionary does not have to contain all the parameters defined by the
+        interface; in case some parameters are omitted, the result list may
+        contain several different implementations.
+        
+        Third is an optional ``include_disabled`` parameter with defaults to
+        ``False``. If ``True``, disabled implementations will be reported as
+        well.
+        
+        An :exc:`~.InternalError` is raised if parameters are passed to the
+        method that are not defined in the interface declaration or not
+        recognized by the interface :meth:`test` method.
         """
         
         if intf_id in self.__intf_index:
@@ -187,11 +201,44 @@ class Registry(object):
             raise InternalError("Unknown interface ID '%s'" % intf_id)
 
     def getImplementationIds(self, intf_id, params=None, include_disabled=False):
+        """
+        This method returns a list of implementation IDs for a given interface.
+        It requires the interface ID as a parameter.
+        
+        Furthermore, a parameter dictionary can be passed to the method. The
+        results will then be filtered according to these parameters. The
+        dictionary does not have to contain all the parameters defined by the
+        interface; in case some parameters are omitted, the result list may
+        contain several different implementations.
+        
+        Third is an optional ``include_disabled`` parameter with defaults to
+        ``False``. If ``True``, disabled implementations will be reported as
+        well.
+        
+        An :exc:`~.InternalError` is raised if parameters are passed to the
+        method that are not defined in the interface declaration or not
+        recognized by the interface :meth:`test` method.
+        """
         impls = self.findImplementations(intf_id, params, include_disabled)
         
         return [impl.__get_impl_id__() for impl in impls]
     
     def getRegistryValues(self, intf_id, registry_key, filter=None, include_disabled=False):
+        """
+        This method returns a list of registry values of implementations 
+        of an interface with ``interface_id`` and a registry key
+        ``registry_key`` defined in the interface declaration.
+        
+        With the ``filter`` argument you can impose certain restrictions on the
+        implementations (registry values) to be returned. It is expected to
+        contain a dictionary of registry keys and values that the implementation
+        must expose to be included.
+        
+        Using the ``include_disabled`` argument you can determine whether.
+        
+        This method raises :exc:`~.InternalError` if the interface ID is
+        unknown.
+        """
         if intf_id in self.__intf_index:
             InterfaceCls = self.__intf_index[intf_id]["intf"]
             
@@ -222,6 +269,12 @@ class Registry(object):
             raise InternalError("Unknown interface ID '%s'" % intf_id)
         
     def getFactoryImplementations(self, factory):
+        """
+        Returns a list of implementations for a given factory.
+        
+        Raises :exc:`~.InternalError` if the factory is not found in the
+        registry.
+        """
         factory_id = factory.__get_impl_id__()
         
         if factory_id in self.__fact_index:
@@ -230,6 +283,42 @@ class Registry(object):
             raise InternalError("Unknown Factory ID '%s'." % factory_id)
 
     def load(self):
+        """
+        This method loads the registry, i.e. it scans the modules specified
+        in the configuration for interfaces and implementations. It is
+        invoked by the ``~.System`` class upon initialization.
+
+        You should *never* invoke this method directly. Always use
+        :meth:`~.System.init` to initialize and :meth:`~.System.getRegistry` to
+        access the registry.
+        
+        There are three configuration settings taken into account.
+        
+        First, the ``system_modules`` setting in ``default.conf``. These
+        modules are always loaded and cannot be left aside in individual
+        instances.
+        
+        Second, the ``module_dirs`` setting in the local configuration of the
+        instance (``eoxserver.conf``) is taken into account. This expected to
+        be a comma-separated list of directories. These directories and all
+        the directory trees underneath them are searched for Python modules.
+        
+        Third, the ``modules`` setting in ``eoxserver.conf``. This is
+        expecte to be a comma-separated list of module names which shall be
+        loaded.
+        
+        All modules specified or detected by scanning directories will be
+        loaded and searched for interfaces descending from
+        :class:`RegisteredInterface` as well as their implementations. These
+        will be automatically included in the registry and accessible using
+        the different binding methods provided.
+        
+        As a last step, the registry is synchronized with the database. This
+        means that the implementation looks up the entries for the different
+        implementations in the database and determines whether they are
+        enabled or not. If it finds an implementation which has not yet been
+        registered it will be saved to the database but disabled by default.
+        """
         # get module directories from config
         reader = RegistryConfigReader(self.config)
         reader.validate()
@@ -256,6 +345,16 @@ class Registry(object):
         self.validate()
     
     def validate(self):
+        """
+        This method is intended to validate the component configuration.
+        
+        It looks up all implementations of :class:`ComponentManagerInterface`
+        and calls their respective :meth:`~ComponentManagerInterface.validate`
+        methods.
+        
+        At the moment, no component managers are implemented, so this
+        method does not have any effects.
+        """
         msgs = []
         
         Managers = self.findImplementations("core.registry.ComponentManager")
@@ -271,29 +370,59 @@ class Registry(object):
             raise ConfigError("\n".join(msgs))
     
     def save(self):
+        """
+        This saves the registry configuration to the database. This means
+        the status of the enabled / disabled flag for each implementation
+        will be saved overriding any previous settings stored.
+        """
         self.validate()
 
         self.__save_to_db()
     
     def getImplementationStatus(self, impl_id):
+        """
+        Returns the implementation status (``True`` for enabled, ``False`` for
+        disabled) for the given implementation ID ``impl_id``.
+        
+        Raises :exc:`~.InternalError` if the implementation ID is
+        unknown.
+        """
         if impl_id in self.__impl_index:
             return self.__impl_index[impl_id]["enabled"]
         else:
             raise InternalError("Unknown implementation ID '%s'" % impl_id)
     
     def enableImplementation(self, impl_id):
+        """
+        Changes the implementation status to enabled for implementation ID
+        ``impl_id``. Note that this change is not automatically stored to
+        the database (you have to call :meth:`save` to do that).
+        
+        Raises :exc:`~.InternalError` if the implementation ID is unknown.
+        """
         if impl_id in self.__impl_index:
             self.__impl_index[impl_id]["enabled"] = True
         else:
             raise InternalError("Unknown implementation ID '%s'" % impl_id)
 
     def disableImplementation(self, impl_id):
+        """
+        Changed the implementation status to disable for implementation ID
+        ``impl_id``. Note that this change is not automatically stored to
+        the database (you have to call :meth:`save` to do that).
+        
+        Raises :exc:`~.InternalError` if the implementation ID is unknown.
+        """
         if impl_id in self.__impl_index:
             self.__impl_index[impl_id]["enabled"] = False
         else:
             raise InternalError("Unknown implementation ID '%s'" % impl_id)
     
     def clone(self):
+        """
+        Returns an exact copy of the registry.
+        """
+        
         registry = Registry(self.config)
         
         registry.__impl_index = self.__impl_index
@@ -656,6 +785,15 @@ class RegisteredInterfaceMetaClass(InterfaceMetaClass):
         pass
 
 class RegisteredInterface(Interface):
+    """
+    This class is the base class for all interfaces to be registered in the
+    registry. All interfaces whose implementations shall be registered must
+    be derived from :class:`RegisteredInterface`.
+    
+    All interfaces derived from :class:`RegisteredInterface` must contain a
+    ``REGISTRY_CONF`` dictionary. See the introduction for details.
+    """
+    
     __metaclass__ = RegisteredInterfaceMetaClass
 
     REGISTRY_CONF = {
@@ -727,6 +865,19 @@ class RegisteredInterface(Interface):
             return []
 
 class TestingInterface(RegisteredInterface):
+    """
+    This class is a descendant of :class:`RegisteredInterface` that adds
+    a single method. It is used for binding by test, which enables binding
+    decisions that cannot easily be implemented by key-value-pair comparisons.
+    
+    .. method:: test(params)
+    
+       This method is invoked by the registry when determining which
+       implementation to bind to. Based on the parameter dictionary ``params``
+       the method shall decide whether the implementation is applicable and
+       return ``True``. If it is not applicable the method shall return
+       ``False``.
+    """
     REGISTRY_CONF = {
         "name": "Registered Testing Interface",
         "intf_id": "core.registry.Testing",
@@ -739,6 +890,28 @@ class TestingInterface(RegisteredInterface):
     )
 
 class FactoryInterface(RegisteredInterface):
+    """
+    This is the basic interface for factories. It is a descendant of
+    :class:`RegisteredInterface`.
+    
+    .. method:: get(**kwargs)
+
+       This method shall return an instance of an implementation that matches
+       the parameters given as keyword arguments. The set of arguments
+       understood depends on the individual factory and can be found in the
+       respective documentation.
+       
+       The method shall raise an exception if no matching implementation or
+       instance thereof can be found, or if the choice is ambiguous.
+    
+    .. method:: find(**kwargs)
+    
+       This method shall return a list of implementation instances that
+       matches the parameters given as keyword arguments. The set of arguments
+       understood depends on the individual factory and can be found in the
+       respective documentation.
+    """
+    
     REGISTRY_CONF = {
         "name": "Registered Factory Interface",
         "intf_id": "core.registry.Factory",
@@ -756,6 +929,11 @@ class FactoryInterface(RegisteredInterface):
     )
     
 class ComponentManagerInterface(RegisteredInterface):
+    """
+    This interface is not in use at the moment. It was intended to provide
+    an API for controlling the status of a larger set of implementations and
+    their dependencies, though the concept has never been elaborated.
+    """
     REGISTRY_CONF = {
         "name": "Component Manager Interface",
         "intf_id": "core.registry.ComponentManager",
@@ -796,13 +974,28 @@ class ComponentManagerInterface(RegisteredInterface):
     )
 
 class RegistryConfigReader(object):
+    """
+    This class provides some functions for reading configuration settings used
+    by the :class:`Registry`. 
+    """
     def __init__(self, config):
         self.config = config
         
     def validate(self):
+        """
+        Validates the configuration; a no-op at the moment.
+        """
         pass
 
     def getSystemModules(self):
+        """
+        This method returns a list of dotted names of system modules. The
+        values are read from ``system_modules`` setting in the
+        ``[core.registry]`` section of the ``default.conf`` configuration file.
+        
+        The format of the setting is expected to be a comma-separated list of
+        the module names.
+        """
         sys_mod_str = self.config.getDefaultConfigValue("core.registry", "system_modules")
         
         if sys_mod_str:
@@ -811,6 +1004,15 @@ class RegistryConfigReader(object):
             return []
     
     def getModuleDirectories(self):
+        """
+        This method returns a list of directory paths where to look for
+        modules to load (see also :meth:`Registry.load`). The values are read
+        from the ``module_dirs`` setting in the ``[core.registry]`` section of
+        the instance specific ``eoxserver.conf`` configuration file.
+        
+        The format of the setting is expected to be a comma-separated list of
+        paths.
+        """
         mod_dir_str = self.config.getConfigValue("core.registry", "module_dirs")
         
         if mod_dir_str:
@@ -819,6 +1021,15 @@ class RegistryConfigReader(object):
             return []
 
     def getModules(self):
+        """
+        This method returs a list of dotted names of modules to be loaded (see
+        also :meth:`Registry.load`). The values are read from the ``modules``
+        setting in the ``[core.registry]`` section of the instance specific
+        ``eoxserver.conf`` configuration file.
+        
+        The format of the setting is expected to be a comma-separated list of
+        dotted module names.
+        """
         mod_str = self.config.getConfigValue("core.registry", "modules")
         
         if mod_str:
