@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: xmltools.py 1518 2012-03-13 16:45:10Z schindlerf $
+# $Id: xmltools.py 2483 2013-05-14 15:13:43Z meissls $
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -61,6 +61,9 @@ except ImportError:
                 except ImportError:
                     pass # could not get etree
 
+
+logger = logging.getLogger(__name__)
+
 #-----------------------------------------------------------------------
 # XML Encoder
 #-----------------------------------------------------------------------
@@ -95,7 +98,7 @@ class XMLEncoder(object):
     
     def _makeElement(self, prefix, tag_name, content):
         """
-        This method creates elements. Itexpects three arguments as
+        This method creates elements. It expects three arguments as
         input:
         
         * the namespace prefix of the element; this can be the empty
@@ -294,7 +297,7 @@ class XPath(object):
             return other
 
     def _getNodesXPath(self, element, xpath):
-        logging.debug("Element: {%s}%s XPath: %s" % (element.namespaceURI, element.localName, str(xpath)))
+        logger.debug("Element: {%s}%s XPath: %s" % (element.namespaceURI, element.localName, str(xpath)))
         
         if len(xpath) == 0:
             return [element]
