@@ -1,6 +1,6 @@
 /*
  *-----------------------------------------------------------------------------
- * $Id: mosaic.py 1296 2012-02-15 14:18:44Z krauses $
+ * $Id: reftools.c 2195 2012-12-19 11:39:32Z schindlerf $
  *
  * Project: EOxServer <http://eoxserver.org>
  * Authors: Stephan Krause <stephan.krause@eox.at>
@@ -35,6 +35,10 @@
 #include <gdal/gdal_alg.h>
 #include <gdal/gdalwarper.h>
 #include <gdal/ogr_srs_api.h>
+#include <gdal/cpl_string.h>
+
+/******************************************************************************/
+/******************************************************************************/
 
 typedef struct {
     size_t n_points;
@@ -57,6 +61,15 @@ typedef struct {
     int y_size;
 } eoxs_rect;
 
+typedef struct {
+    size_t x_size;
+    size_t y_size;
+    double geotransform[6];
+} eoxs_image_info;
+
+/******************************************************************************/
+/******************************************************************************/
+
 void eoxs_destroy_footprint(eoxs_footprint *fp) {
     free(fp->x);
     free(fp->y);
@@ -68,23 +81,24 @@ void eoxs_free_string(char* str)
     free(str);
 }
 
-void *eoxs_get_referenceable_grid_transformer(GDALDatasetH ds) {
+void *eoxs_create_referenceable_grid_transformer(GDALDatasetH ds, int order) {
     int gcp_count;
     const GDAL_GCP *gcps;
-    
+
     if (!ds) return NULL;
     
     gcp_count = GDALGetGCPCount(ds);
     gcps = GDALGetGCPs(ds);
-    
-    return GDALCreateTPSTransformer(gcp_count, gcps, FALSE);
+
+    if (order == -1) {
+        return GDALCreateTPSTransformer(gcp_count, gcps, FALSE);
+    }
+    else {
+        return GDALCreateGCPTransformer(gcp_count, gcps, order, FALSE);
+    }
 }
 
-void eoxs_destroy_referenceable_grid_transformer(void *transformer) {
-    GDALDestroyTPSTransformer(transformer);
-}
-
-eoxs_footprint *eoxs_calculate_footprint(GDALDatasetH ds) {
+CPLErr eoxs_calculate_footprint(GDALDatasetH ds, int order, eoxs_footprint **out_footprint) {
     void *transformer;
     int x_size, y_size;
     double *x, *y, *z;
@@ -94,19 +108,20 @@ eoxs_footprint *eoxs_calculate_footprint(GDALDatasetH ds) {
     int *success;
     int i;
 
-    eoxs_footprint *ret;
-
-    if (!ds) 
-    {
-        fprintf( stderr, "ERROR: %s:%i : Invalid GDAL dataset!",__FILE__,__LINE__ ) ; 
-        return NULL;
+    if (!ds) {
+        CPLError(CE_Failure, CPLE_ObjectNull, "No dataset passed.");
+        return CE_Failure;
     }
 
-    if (( 0 == GDALGetGCPCount(ds) )||( '\0' == GDALGetGCPProjection(ds)[0] ))
-    { 
-        fprintf( stderr, "ERROR: %s:%i : The GDAL dataset has no GCP!",__FILE__,__LINE__ ) ; 
-        return NULL ; 
-    } 
+    else if ( 0 == GDALGetGCPCount(ds) ) {
+        CPLError(CE_Failure, CPLE_IllegalArg, "The given dataset has no GCPs.");
+        return CE_Failure; 
+    }
+
+    else if ( '\0' == GDALGetGCPProjection(ds)[0] ) {
+        CPLError(CE_Failure, CPLE_IllegalArg, "The given dataset has no GCP projection.");
+        return CE_Failure; 
+    }
 
 #ifdef DEBUG 
     { // debug - print the GCPs 
@@ -134,12 +149,13 @@ eoxs_footprint *eoxs_calculate_footprint(GDALDatasetH ds) {
     x_size = GDALGetRasterXSize(ds);
     y_size = GDALGetRasterYSize(ds);
 
-    transformer = eoxs_get_referenceable_grid_transformer(ds);
+    transformer = eoxs_create_referenceable_grid_transformer(ds, order);
 
-    if (!transformer)
-    {
-        fprintf( stderr, "ERROR: %s:%i : Failed to create GCP transformer!",__FILE__,__LINE__ ) ; 
-        return NULL;
+    if (!transformer) {
+        if (CPLGetLastErrorMsg() == NULL) {
+            CPLError(CE_Failure, CPLE_OutOfMemory, "Failed to create GCP transformer.");
+        }
+        return CE_Failure; 
     }
 
     x_e = x_size / 100 - 1; if (x_e < 0) x_e = 0;
@@ -176,17 +192,18 @@ eoxs_footprint *eoxs_calculate_footprint(GDALDatasetH ds) {
         z[x_e+1+y_e+1+x_e+1+i] = 0;
     }
 
-    GDALTPSTransform(transformer, FALSE, n_points, x, y, z, success);
+    /* for TPS and GCP methods returns always true */
+    GDALUseTransformer(transformer, FALSE, n_points, x, y, z, success);
 
     // discard unused information
     free(z);
     free(success);
-    eoxs_destroy_referenceable_grid_transformer(transformer);
+    GDALDestroyTransformer(transformer);
 
-    ret = malloc(sizeof(eoxs_footprint));
-    ret->n_points = n_points;
-    ret->x = x;
-    ret->y = y;
+    *out_footprint = malloc(sizeof(eoxs_footprint));
+    (*out_footprint)->n_points = n_points;
+    (*out_footprint)->x = x;
+    (*out_footprint)->y = y;
 
 #ifdef DEBUG 
     { // debug - print the calculated footprint
@@ -202,36 +219,47 @@ eoxs_footprint *eoxs_calculate_footprint(GDALDatasetH ds) {
     }
 #endif /* DEBUG */
 
-    return ret;
+    return CE_None;
 }
 
-const char *eoxs_get_footprint_wkt(GDALDatasetH ds) {
+CPLErr eoxs_get_footprint_wkt(GDALDatasetH ds, int order, char **out_wkt) {
     eoxs_footprint *fp;
-    char *wkt, buffer[256];
-    int i;
-    fp = eoxs_calculate_footprint(ds);
+    char buffer[512];
+    int i, maxlen;
+    CPLErr ret;
+    *out_wkt = NULL;
 
-    if (!fp) return NULL;
-
-    wkt = calloc((fp->n_points + 1) * 100 + sizeof("POLYGON(())"), sizeof(char));
-    sprintf(wkt, "POLYGON((");
-
-    for (i=0; i<fp->n_points; ++i) {
-        sprintf(buffer, "%f %f", fp->x[i], fp->y[i]);
-        if(i != 0) {
-            strcat(wkt, ",");
-        }
-        strcat(wkt, buffer);
+    if ((ret = eoxs_calculate_footprint(ds, order, &fp)) != CE_None) {
+        return ret;
     }
 
-    sprintf(buffer, ",%f %f", fp->x[0], fp->y[0]);
-    strcat(wkt, buffer);
-    strcat(wkt, "))");
+    maxlen = (fp->n_points + 1) * 100 + sizeof("POLYGON(())");
+
+    *out_wkt = calloc(maxlen, sizeof(char));
+
+    if (!*out_wkt) {
+        eoxs_destroy_footprint(fp);
+        CPLError(CE_Failure, CPLE_OutOfMemory, "Error allocating memory.");
+        return CE_Failure;
+    }
+    snprintf(*out_wkt, maxlen, "POLYGON((");
+
+    for (i = 0; i < fp->n_points; ++i) {
+        snprintf(buffer, sizeof(buffer), "%f %f", fp->x[i], fp->y[i]);
+        if(i != 0) {
+            CPLStrlcat(*out_wkt, ",", maxlen);
+        }
+        CPLStrlcat(*out_wkt, buffer, maxlen);
+    }
+
+    snprintf(buffer, sizeof(buffer), ",%f %f", fp->x[0], fp->y[0]);
+    CPLStrlcat(*out_wkt, buffer, maxlen);
+    CPLStrlcat(*out_wkt, "))", maxlen);
 
     // clean up
     eoxs_destroy_footprint(fp);
 
-    return wkt;
+    return CE_None;
 }
 
 double eoxs_array_min(int n, double *c) {
@@ -279,7 +307,7 @@ void eoxs_get_intermediate_point_count(
     x[2] = (double) ds_x_size; y[2] = (double) ds_y_size; z[2] = 0;
     x[3] = 0; y[3] = (double) ds_y_size; z[3] = 0;
     
-    GDALTPSTransform(transformer, FALSE, 4, x, y, z, success);
+    GDALUseTransformer(transformer, FALSE, 4, x, y, z, success);
     
     dist = MIN(
         (eoxs_array_max(4, x) - eoxs_array_min(4, x)) / (double) (ds_x_size / 100),
@@ -297,7 +325,7 @@ void eoxs_get_intermediate_point_count(
     *n_y = (int) ceil((eoxs_array_max(4, y) - eoxs_array_min(4, y)) / dist);
 }
 
-int eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, eoxs_rect *out_rect) {
+CPLErr eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, int order, eoxs_rect *out_rect) {
     void *transformer;
     
     OGRSpatialReferenceH gcp_srs, subset_srs;
@@ -315,17 +343,20 @@ int eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, eoxs_rect *out_r
     int minx, miny, maxx, maxy;
     int i;
     
-    if (!ds) 
-    {
-        fprintf( stderr, "ERROR: %s:%i : Invalid GDAL dataset!",__FILE__,__LINE__ ) ; 
-        return 0;
+    if (!ds) {
+        CPLError(CE_Failure, CPLE_ObjectNull, "No dataset passed.");
+        return CE_Failure;
     }
 
-    if (( 0 == GDALGetGCPCount(ds) )||( '\0' == GDALGetGCPProjection(ds)[0] ))
-    { 
-        fprintf( stderr, "ERROR: %s:%i : The GDAL dataset has no GCP!",__FILE__,__LINE__ ) ; 
-        return 0 ; 
-    } 
+    else if ( 0 == GDALGetGCPCount(ds) ) {
+        CPLError(CE_Failure, CPLE_IllegalArg, "The given dataset has no GCPs.");
+        return CE_Failure; 
+    }
+
+    else if ( '\0' == GDALGetGCPProjection(ds)[0] ) {
+        CPLError(CE_Failure, CPLE_IllegalArg, "The given dataset has no GCP projection.");
+        return CE_Failure; 
+    }
 
 #ifdef DEBUG 
     { // debug - print the GCPs  
@@ -351,11 +382,13 @@ int eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, eoxs_rect *out_r
     ds_x_size = GDALGetRasterXSize(ds);
     ds_y_size = GDALGetRasterYSize(ds);
     
-    transformer = eoxs_get_referenceable_grid_transformer(ds);
+    transformer = eoxs_create_referenceable_grid_transformer(ds, order);
     
     if (!transformer) {
-        fprintf( stderr, "ERROR: %s:%i : Failed to create GCP transformer!",__FILE__,__LINE__ ) ; 
-        return 0;
+        if (CPLGetLastErrorMsg() == NULL) {
+            CPLError(CE_Failure, CPLE_OutOfMemory, "Failed to create GCP transformer.");
+        }
+        return CE_Failure; 
     }
     
     gcp_srs = OSRNewSpatialReference( GDALGetGCPProjection(ds) );
@@ -364,11 +397,13 @@ int eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, eoxs_rect *out_r
     
     ct = OCTNewCoordinateTransformation(subset_srs, gcp_srs);
     if (!ct) {
-        fprintf(stderr, "ERROR: %s:%i : Failed to create coordinate trasnformation!",__FILE__,__LINE__ ) ; 
-        eoxs_destroy_referenceable_grid_transformer(transformer);
+        if (CPLGetLastErrorMsg() == NULL) {
+            CPLError(CE_Failure, CPLE_OutOfMemory, "Failed to create coordinate transformer.");
+        }
+        GDALDestroyTransformer(transformer);
         OSRDestroySpatialReference( gcp_srs ); 
         OSRDestroySpatialReference( subset_srs ); 
-        return 0;
+        return CE_Failure;
     }
     
     eoxs_get_intermediate_point_count(&n_x, &n_y, ds_x_size, ds_y_size, subset, transformer, ct);
@@ -426,7 +461,7 @@ int eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, eoxs_rect *out_r
 #endif /* DEBUG */
 
     OCTTransform(ct, n_points, x, y, z);
-    GDALTPSTransform(transformer, TRUE, n_points, x, y, z, success);
+    GDALUseTransformer(transformer, TRUE, n_points, x, y, z, success);
     
     minx = (int) floor(eoxs_array_min(n_points, x));
     maxx = (int) ceil(eoxs_array_max(n_points, x));
@@ -439,15 +474,15 @@ int eoxs_rect_from_subset(GDALDatasetH ds, eoxs_subset *subset, eoxs_rect *out_r
     out_rect->y_size = maxy - miny + 1;
     
     free(x); free(y); free(z); free(success);
-    eoxs_destroy_referenceable_grid_transformer(transformer);
+    GDALDestroyTransformer(transformer);
     OCTDestroyCoordinateTransformation( ct ); 
     OSRDestroySpatialReference( gcp_srs ); 
     OSRDestroySpatialReference( subset_srs ); 
     
-    return 1;
+    return CE_None;
 }
 
-int eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int srid) {
+CPLErr eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int srid) {
     GDALDatasetH vrt_ds;
     OGRSpatialReferenceH dst_srs;
     char *dst_srs_wkt;
@@ -456,7 +491,10 @@ int eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int sri
     void *transformer;
     GDALWarpOptions *warp_options;
     
-    if (!ds) return 0;
+    if (!ds) {
+        CPLError(CE_Failure, CPLE_ObjectNull, "No dataset passed.");
+        return CE_Failure;
+    }
 
     if (srid != 0) {
         dst_srs = OSRNewSpatialReference("");
@@ -480,8 +518,11 @@ int eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int sri
     );
     
     if (!transformer) {
+        if (CPLGetLastErrorMsg() == NULL) {
+            CPLError(CE_Failure, CPLE_OutOfMemory, "Failed to create image projection transformer.");
+        }
         if (free_dst_srs_wkt) free(dst_srs_wkt);
-        return 0;
+        return CE_Failure;
     }
     
     warp_options = GDALCreateWarpOptions();
@@ -498,10 +539,13 @@ int eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int sri
     );
     
     if (!vrt_ds) {
+        if (CPLGetLastErrorMsg() == NULL) {
+            CPLError(CE_Failure, CPLE_FileIO, "Failed to created warped VRT.");
+        }
         GDALDestroyGenImgProjTransformer(transformer);
         GDALDestroyWarpOptions(warp_options);
         if (free_dst_srs_wkt) free(dst_srs_wkt);
-        return 0;
+        return CE_Failure;
     }
     
     GDALSetDescription(vrt_ds, vrt_filename);
@@ -512,5 +556,188 @@ int eoxs_create_rectified_vrt(GDALDatasetH ds, const char *vrt_filename, int sri
     GDALDestroyWarpOptions(warp_options);
     if (free_dst_srs_wkt) free(dst_srs_wkt);
     
-    return 1;
+    return CE_None;
+}
+
+
+/* Thin wrapper for GDALSuggestedWarpOutput which allows the setting of the order. */
+CPLErr eoxs_suggested_warp_output(GDALDatasetH ds, 
+                                  const char *src_wkt, /* can be NULL */
+                                  const char *dst_wkt,
+                                  int order,
+                                  eoxs_image_info* out) {
+
+    CPLErr ret;
+    void *transformer = GDALCreateGenImgProjTransformer(
+        ds,
+        src_wkt,
+        NULL,
+        dst_wkt,
+        TRUE,
+        0.0,
+        order
+    );
+
+    if (!transformer) {
+        return CE_Failure;
+    }
+    
+    ret = GDALSuggestedWarpOutput(ds, GDALGenImgProjTransform, 
+                                  transformer, 
+                                  out->geotransform,
+                                  (int*) &out->x_size, (int*) &out->y_size);
+
+    // clean up
+    GDALDestroyTransformer(transformer);
+
+    return ret;
+}
+
+
+/* copied from gdalwarper.cpp */
+CPLErr eoxs_reproject_image(GDALDatasetH hSrcDS,
+                            const char *pszSrcWKT, 
+                            GDALDatasetH hDstDS,
+                            const char *pszDstWKT,
+                            GDALResampleAlg eResampleAlg, 
+                            double dfWarpMemoryLimit, 
+                            double dfMaxError,
+                            int order) {
+    
+    GDALWarpOptions *psWOptions;
+
+/* -------------------------------------------------------------------- */
+/*      Setup a reprojection based transformer.                         */
+/* -------------------------------------------------------------------- */
+    void *hTransformArg;
+
+    hTransformArg = 
+        GDALCreateGenImgProjTransformer(
+            hSrcDS,
+            pszSrcWKT,
+            hDstDS,
+            pszDstWKT,
+            TRUE,
+            0.0,
+            order
+        );
+
+    if( hTransformArg == NULL )
+        return CE_Failure;
+
+/* -------------------------------------------------------------------- */
+/*      Create a copy of the user provided options, or a defaulted      */
+/*      options structure.                                              */
+/* -------------------------------------------------------------------- */
+    psWOptions = GDALCreateWarpOptions();
+    psWOptions->eResampleAlg = eResampleAlg;
+
+/* -------------------------------------------------------------------- */
+/*      Set transform.                                                  */
+/* -------------------------------------------------------------------- */
+    if( dfMaxError > 0.0 )
+    {
+        psWOptions->pTransformerArg = 
+            GDALCreateApproxTransformer( GDALGenImgProjTransform, 
+                                         hTransformArg, dfMaxError );
+
+        psWOptions->pfnTransformer = GDALApproxTransform;
+    }
+    else
+    {
+        psWOptions->pfnTransformer = GDALGenImgProjTransform;
+        psWOptions->pTransformerArg = hTransformArg;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Set file and band mapping.                                      */
+/* -------------------------------------------------------------------- */
+    int  iBand;
+
+    psWOptions->hSrcDS = hSrcDS;
+    psWOptions->hDstDS = hDstDS;
+
+    if( psWOptions->nBandCount == 0 )
+    {
+        psWOptions->nBandCount = MIN(GDALGetRasterCount(hSrcDS),
+                                     GDALGetRasterCount(hDstDS));
+        
+        psWOptions->panSrcBands = (int *) 
+            CPLMalloc(sizeof(int) * psWOptions->nBandCount);
+        psWOptions->panDstBands = (int *) 
+            CPLMalloc(sizeof(int) * psWOptions->nBandCount);
+
+        for( iBand = 0; iBand < psWOptions->nBandCount; iBand++ )
+        {
+            psWOptions->panSrcBands[iBand] = iBand+1;
+            psWOptions->panDstBands[iBand] = iBand+1;
+        }
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Set source nodata values if the source dataset seems to have    */
+/*      any.                                                            */
+/* -------------------------------------------------------------------- */
+    for( iBand = 0; iBand < psWOptions->nBandCount; iBand++ )
+    {
+        GDALRasterBandH hBand = GDALGetRasterBand( hSrcDS, iBand+1 );
+        int             bGotNoData = FALSE;
+        double          dfNoDataValue;
+
+        if (GDALGetRasterColorInterpretation(hBand) == GCI_AlphaBand)
+        {
+            psWOptions->nSrcAlphaBand = iBand + 1;
+        }
+
+        dfNoDataValue = GDALGetRasterNoDataValue( hBand, &bGotNoData );
+        if( bGotNoData )
+        {
+            if( psWOptions->padfSrcNoDataReal == NULL )
+            {
+                int  ii;
+
+                psWOptions->padfSrcNoDataReal = (double *) 
+                    CPLMalloc(sizeof(double) * psWOptions->nBandCount);
+                psWOptions->padfSrcNoDataImag = (double *) 
+                    CPLMalloc(sizeof(double) * psWOptions->nBandCount);
+
+                for( ii = 0; ii < psWOptions->nBandCount; ii++ )
+                {
+                    psWOptions->padfSrcNoDataReal[ii] = -1.1e20;
+                    psWOptions->padfSrcNoDataImag[ii] = 0.0;
+                }
+            }
+
+            psWOptions->padfSrcNoDataReal[iBand] = dfNoDataValue;
+        }
+
+        hBand = GDALGetRasterBand( hDstDS, iBand+1 );
+        if (hBand && GDALGetRasterColorInterpretation(hBand) == GCI_AlphaBand)
+        {
+            psWOptions->nDstAlphaBand = iBand + 1;
+        }
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Create a warp options based on the options.                     */
+/* -------------------------------------------------------------------- */
+    GDALWarpOperationH  hWarper = GDALCreateWarpOperation( psWOptions );
+    CPLErr eErr = CE_Failure;
+
+    if( hWarper )
+        eErr = GDALChunkAndWarpImage( hWarper, 0, 0, 
+                                      GDALGetRasterXSize(hDstDS),
+                                      GDALGetRasterYSize(hDstDS) );
+
+/* -------------------------------------------------------------------- */
+/*      Cleanup.                                                        */
+/* -------------------------------------------------------------------- */
+    GDALDestroyGenImgProjTransformer( hTransformArg );
+
+    if( dfMaxError > 0.0 )
+        GDALDestroyApproxTransformer( psWOptions->pTransformerArg );
+        
+    GDALDestroyWarpOptions( psWOptions );
+
+    return eErr;
 }
