@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: metadata.py 1576 2012-03-23 16:03:20Z meissls $
+# $Id: metadata.py 2240 2013-02-19 19:56:56Z schindlerf $
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -36,16 +36,16 @@ import re
 import datetime
 from os.path import splitext
 
-from osgeo import gdal
 from django.contrib.gis.geos import GEOSGeometry
 
+from eoxserver.contrib import gdal
 from eoxserver.core.system import System
 from eoxserver.core.exceptions import (
     InternalError, ImplementationAmbiguous, ImplementationNotFound
 )
-from eoxserver.core.util.xmltools import XMLDecoder, etree
+from eoxserver.core.util.xmltools import XMLDecoder, XMLEncoder, etree
 from eoxserver.core.util.timetools import getDateTime
-from eoxserver.core.util.geotools import posListToWkt
+from eoxserver.resources.coverages import crss
 from eoxserver.resources.coverages.interfaces import (
     GenericEOMetadataInterface, EOMetadataFormatInterface,
     EOMetadataReaderInterface
@@ -274,6 +274,21 @@ class XMLEOMetadataFormat(XMLMetadataFormat):
         return getDateTime(decoder.getValue("endtime"))
     
     def _get_footprint(self, decoder):
+
+        #footprint SRID 
+        srid = 4326
+        
+        # float format 
+        frm  = "%.10g %.10g"
+
+        #axes swapper 
+        swap = crss.getAxesSwapper( srid )
+
+        # WKT polygon packing closure 
+        def posListToWkt( pl ):
+            tmp = [ frm%swap(pl[i],pl[i+1]) for i in xrange(0,len(pl),2) ]
+            return ",".join( tmp )
+
         polygon_dicts = decoder.getValue("footprint")
         
         polygon_wkts = []
@@ -651,3 +666,29 @@ class DatasetMetadataFileReader(object):
 
 DatasetMetadataFileReaderImplementation = \
 EOMetadataReaderInterface.implement(DatasetMetadataFileReader)
+
+
+class NativeMetadataFormatEncoder(XMLEncoder):
+    """
+    Encodes EO Coverage metadata 
+    """
+    
+    def encodeMetadata(self, eoid, begin_time, end_time, polygon):
+        return self._makeElement("", "Metadata", [
+            ("", "EOID", eoid),
+            ("", "BeginTime", begin_time),
+            ("", "EndTime", end_time),
+            ("", "Footprint", [
+                ("", "Polygon", [
+                    ("", "Exterior", self._posListToString(polygon[0]))
+                ] + [
+                    tuple("", "Interior", self._posListToString(interior)) 
+                    for interior in polygon[1:]
+                ])
+            ])
+        ])
+    
+    
+    def _posListToString(self, ring):
+        return " ".join(map(str, ring))
+
