@@ -1,9 +1,10 @@
 #-------------------------------------------------------------------------------
-# $Id: models.py 2147 2012-11-28 11:17:33Z schindlerf $
+# $Id: models.py 2444 2013-04-26 17:45:06Z meissls $
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
 #          Stephan Meissl <stephan.meissl@eox.at>
+#          Martin Paces <martin.paces@eox.at>
 #
 #-------------------------------------------------------------------------------
 # Copyright (C) 2011 EOX IT Services GmbH
@@ -28,15 +29,16 @@
 #-------------------------------------------------------------------------------
 
 import re
+import logging
 
-from osgeo import gdalconst
 from django.core.validators import RegexValidator
 from django.core.exceptions import ValidationError
 
 from django.contrib.gis.db import models
-from django.contrib.gis.geos.polygon import Polygon
+from django.contrib.gis.geos import Point, LinearRing, Polygon, MultiPolygon 
 from django.contrib.gis.geos.error import GEOSException
 
+from eoxserver.contrib import gdal
 from eoxserver.core.models import Resource
 from eoxserver.backends.models import (
     Location, LocalPath, RemotePath,
@@ -46,6 +48,12 @@ from eoxserver.resources.coverages.validators import (
     validateEOOM, validateCoverageIDnotInEOOM
 )
 
+from eoxserver.resources.coverages.formats import _gerexValMime as regexMIMEType
+
+
+logger = logging.getLogger(__name__)
+
+MIMETypeValidator = RegexValidator( regexMIMEType , message="The field must contain a valid MIME Type!" ) 
 NCNameValidator = RegexValidator(re.compile(r'^[a-zA-z_][a-zA-Z0-9_.-]*$'), message="This field must contain a valid NCName.")
 
 class NilValueRecord(models.Model):
@@ -75,23 +83,8 @@ class BandRecord(models.Model):
     definition = models.CharField(max_length=256)
     uom = models.CharField("UOM", max_length=16)
     nil_values = models.ManyToManyField(NilValueRecord, null=True, blank=True, verbose_name="Nil Value")
-    gdal_interpretation = models.IntegerField("GDAL Interpretation", default=gdalconst.GCI_Undefined,
-        choices=(
-            (gdalconst.GCI_Undefined, 'Undefined'),
-            (gdalconst.GCI_GrayIndex, 'Grayscale'),
-            (gdalconst.GCI_PaletteIndex, 'Palette Index'),
-            (gdalconst.GCI_RedBand, 'Red'),
-            (gdalconst.GCI_GreenBand, 'Green'),
-            (gdalconst.GCI_BlueBand, 'Blue'),
-            (gdalconst.GCI_AlphaBand, 'Alpha'),
-            (gdalconst.GCI_HueBand, 'Hue'),
-            (gdalconst.GCI_SaturationBand, 'Saturation'),
-            (gdalconst.GCI_LightnessBand, 'Lightness'),
-            (gdalconst.GCI_CyanBand, 'Cyan'),
-            (gdalconst.GCI_MagentaBand, 'Magenta'),
-            (gdalconst.GCI_YellowBand, 'Yellow'),
-            (gdalconst.GCI_BlackBand, 'Black')
-        )
+    gdal_interpretation = models.IntegerField("GDAL Interpretation", default=gdal.GCI_Undefined,
+        choices=gdal.GCI_TO_NAME.items()
     )
 
     def __unicode__(self):
@@ -103,19 +96,7 @@ class BandRecord(models.Model):
 
 class RangeTypeRecord(models.Model):
     name = models.CharField(max_length=256)
-    data_type = models.IntegerField(choices=(
-        (gdalconst.GDT_Byte, 'Byte'),
-        (gdalconst.GDT_UInt16, 'Unsigned Integer 16-bit'),
-        (gdalconst.GDT_Int16, 'Signed Integer 16-bit'),
-        (gdalconst.GDT_UInt32, 'Unsigned Integer 32-bit'),
-        (gdalconst.GDT_Int32, 'Signed Integer 32-bit'),
-        (gdalconst.GDT_Float32, 'Floating Point 32-bit'),
-        (gdalconst.GDT_Float64, 'Floating Point 64-bit'),
-        (gdalconst.GDT_CInt16, 'GDAL CInt 16-bit'),
-        (gdalconst.GDT_CInt32, 'GDAL CInt 32-bit'),
-        (gdalconst.GDT_CFloat32, 'GDAL CFloat 32-bit'),
-        (gdalconst.GDT_CFloat64, 'GDAL CFloat 64-bit')
-    ))
+    data_type = models.IntegerField(choices=gdal.GDT_TO_NAME.items())
     bands = models.ManyToManyField(BandRecord, through="RangeType2Band")
 
     def __unicode__(self):
@@ -133,11 +114,6 @@ class RangeType2Band(models.Model):
         verbose_name = "Band in Range type"
 
 class ExtentRecord(models.Model):
-    def __unicode__(self):
-        return "Extent (SRID=%d; %f, %f, %f, %f; Size: %d x %d)" % (
-            self.srid, self.minx, self.miny, self.maxx, self.maxy, self.size_x, self.size_y
-        )
-
     srid = models.IntegerField("SRID")
     size_x = models.IntegerField()
     size_y = models.IntegerField()
@@ -146,6 +122,11 @@ class ExtentRecord(models.Model):
     maxx = models.FloatField()
     maxy = models.FloatField()
 
+    def __unicode__(self):
+        return "Extent (SRID=%d; %f, %f, %f, %f; Size: %d x %d)" % (
+            self.srid, self.minx, self.miny, self.maxx, self.maxy, self.size_x, self.size_y
+        )
+    
     class Meta:
         verbose_name = "Extent"
 
@@ -238,13 +219,15 @@ class LocalDataPackage(DataPackage):
     
     data_location = models.ForeignKey(LocalPath, related_name="data_file_packages")
     metadata_location = models.ForeignKey(LocalPath, related_name="metadata_file_packages", null=True)
+    source_format = models.CharField( max_length=64, null=False, blank=False, validators = [ MIMETypeValidator ] ) 
 
 class RemoteDataPackage(DataPackage):
     DATA_PACKAGE_TYPE = "remote"
     
     data_location = models.ForeignKey(RemotePath, related_name="data_file_packages")
     metadata_location = models.ForeignKey(RemotePath, related_name="metadata_file_packages", null=True)
-    
+    source_format = models.CharField( max_length=64, null=False, blank=False, validators = [ MIMETypeValidator ] ) 
+
     cache_file = models.ForeignKey(CacheFile, related_name="remote_data_packages", null=True)
     
     def delete(self):
@@ -282,6 +265,9 @@ class CoverageRecord(Resource):
         super(CoverageRecord, self).clean()
         if self.automatic and self.data_source is None:
             raise ValidationError('DataSource has to be set if automatic is true.')
+    
+    def __unicode__(self):
+        return self.coverage_id
 
 class PlainCoverageRecord(CoverageRecord):
     extent = models.ForeignKey(ExtentRecord, related_name = "single_file_coverages")
@@ -328,6 +314,70 @@ class EODatasetMixIn(EOCoverageMixIn):
         super(EODatasetMixIn, self).delete()
         data_package.delete()
         
+def _checkFootprint(footprint, extent):
+    """ Check footprint to match the extent. """
+    
+    #TODO: Make the rtol value configurable. 
+    # allow footprint to exceed extent by given % of smaller extent size
+    rtol = 0.005 # .5% 
+    difx = abs(extent.maxx - extent.minx)
+    dify = abs(extent.maxy - extent.miny)
+    atol = rtol * min(difx, dify) 
+    
+    try:
+        bbox = Polygon.from_bbox((extent.minx, extent.miny, extent.maxx, extent.maxy))
+        bbox.srid = int(extent.srid)
+        
+        bbox_ll = bbox.transform(footprint.srs, clone=True)
+        
+        normalized_space = Polygon.from_bbox((-180, -90, 180, 90))
+        non_normalized_space = Polygon.from_bbox((180, -90, 360, 90))
+        
+        normalized_space.srid = int(extent.srid)
+        non_normalized_space.srid = int(extent.srid)
+        
+        if not normalized_space.contains(bbox_ll):
+            # create 2 bboxes for each side of the date line
+            bbox_ll1 = bbox_ll.intersection(normalized_space)
+            bbox_ll2 = bbox_ll.intersection(non_normalized_space)
+            
+            bbox_ll2 = Polygon(LinearRing([Point(x - 360, y) for x, y in bbox_ll2.exterior_ring]), srid=bbox_ll2.srid)
+            
+            bbox_ll1.transform(extent.srid)
+            bbox_ll2.transform(extent.srid)
+            
+            e1 = bbox_ll1.extent
+            e2 = bbox_ll2.extent
+            
+            bbox = MultiPolygon(
+                Polygon.from_bbox((e1[0] - atol, e2[1] - atol, e1[2] + atol, e1[3] + atol)),
+                Polygon.from_bbox((e2[0] - atol, e2[1] - atol, e2[2] + atol, e2[3] + atol)),
+                srid=extent.srid
+            )
+        else:
+            # just use the tolerance for a slightly larger bbox
+            bbox = Polygon.from_bbox((extent.minx - atol, extent.miny - atol,
+                                      extent.maxx + atol, extent.maxy + atol))
+            bbox.srid = int(extent.srid)
+        
+        if footprint.srid != bbox.srid:
+            footprint_bboxsrs = footprint.transform(bbox.srs, clone=True)
+        else:
+            footprint_bboxsrs = footprint
+        
+        logger.debug("Extent: %s" % bbox.wkt)
+        logger.debug("Footprint: %s" % footprint_bboxsrs.wkt)
+        
+        if not bbox.contains(footprint_bboxsrs):
+            raise ValidationError("The datasets's extent does not surround its"
+                " footprint. Extent: '%s' Footprint: '%s'."
+                % (bbox.wkt, footprint_bboxsrs.wkt)
+            )
+        
+    except GEOSException: 
+        pass
+
+
 class RectifiedDatasetRecord(CoverageRecord, EODatasetMixIn):
     extent = models.ForeignKey(ExtentRecord, related_name="rect_datasets")
     
@@ -340,22 +390,7 @@ class RectifiedDatasetRecord(CoverageRecord, EODatasetMixIn):
         
         # TODO: this does not work in the admins changelist.save method
         # A wrong WKB is inside the eo_metadata.footprint entry
-        try:
-            footprint = self.eo_metadata.footprint
-            EPSILON = abs((self.extent.maxx - self.extent.minx) / self.extent.size_x)
-            bbox = Polygon.from_bbox((self.extent.minx - EPSILON,
-                                     self.extent.miny - EPSILON,
-                                     self.extent.maxx + EPSILON,
-                                     self.extent.maxy + EPSILON)) # TODO: Adjust according to axis order of SRID.
-            bbox.set_srid(int(self.extent.srid))
-            
-            if footprint.srid != bbox.srid:
-                footprint.transform(bbox.srs)
-            
-            if not bbox.contains(footprint):
-                raise ValidationError("Extent does not surround footprint. Extent: '%s' Footprint: '%s'" % (str(bbox), str(footprint)))
-        except GEOSException:
-            pass
+        _checkFootprint( self.eo_metadata.footprint , self.extent ) 
         
         validateCoverageIDnotInEOOM(self.coverage_id, self.eo_metadata.eo_gml)
     
@@ -365,8 +400,7 @@ class RectifiedDatasetRecord(CoverageRecord, EODatasetMixIn):
         extent.delete()
 
 class ReferenceableDatasetRecord(CoverageRecord, EODatasetMixIn):
-    size_x = models.IntegerField()
-    size_y = models.IntegerField()
+    extent = models.ForeignKey(ExtentRecord, related_name="refa_datasets")
     
     class Meta:
         verbose_name = "Referenceable Dataset"
@@ -374,8 +408,19 @@ class ReferenceableDatasetRecord(CoverageRecord, EODatasetMixIn):
         
     def clean(self):
         super(ReferenceableDatasetRecord, self).clean()
+        
+        # TODO: taken from Rectified DS, check if applicable to Referenceable DS too
+        # TODO: this does not work in the admins changelist.save method
+        # A wrong WKB is inside the eo_metadata.footprint entry
+        _checkFootprint( self.eo_metadata.footprint , self.extent ) 
+
         validateCoverageIDnotInEOOM(self.coverage_id, self.eo_metadata.eo_gml)
 
+    def delete(self):
+        extent = self.extent
+        super(EOCoverageMixIn, self).delete()
+        extent.delete()
+
 class RectifiedStitchedMosaicRecord(CoverageRecord, EOCoverageMixIn):
     extent = models.ForeignKey(ExtentRecord, related_name="rect_stitched_mosaics")
     data_sources = models.ManyToManyField(DataSource,
@@ -441,6 +486,8 @@ class DatasetSeriesRecord(Resource):
                                           blank=True, null=True,
                                           related_name="dataset_series_set",
                                           verbose_name="Referenceable Dataset(s)")
+    
+    layer_metadata = models.ManyToManyField(LayerMetadataRecord, null=True, blank=True)
 
     def __unicode__(self):
         return self.eo_id
