@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: getcap.py 1827 2012-07-11 10:39:31Z schindlerf $
+# $Id: getcap.py 2453 2013-05-05 12:11:15Z meissls $
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -27,17 +27,39 @@
 # THE SOFTWARE.
 #-------------------------------------------------------------------------------
 
+"""
+This module provides handlers for WCS 2.0 / EO-WCS GetCapabilities requests.
+"""
+
 from xml.dom import minidom
 
 from eoxserver.core.system import System
 from eoxserver.core.util.xmltools import DOMtoXML
 from eoxserver.core.exceptions import InternalError
 from eoxserver.services.owscommon import OWSCommonConfigReader
-from eoxserver.services.ows.wcs.common import WCSCommonHandler
+from eoxserver.services.ows.wcs.common import WCSCommonHandler, \
+            getMSOutputFormatsAll,getMSWCSFormatMD,getMSWCSSRSMD
 from eoxserver.services.ows.wcs.encoders import WCS20EOAPEncoder
+from eoxserver.services.ows.wcs.wcs20.common import WCS20ConfigReader
 from eoxserver.services.ows.wcst.wcst11AlterCapabilities import wcst11AlterCapabilities20
 
+
 class WCS20GetCapabilitiesHandler(WCSCommonHandler):
+    """
+    This is the handler for WCS 2.0 / EO-WCS GetCapabilities requests. It
+    inherits from :class:`~.WCSCommonHandler`.
+    
+    As for all handlers, the entry point is the
+    :meth:`~.WCSCommonHandler.handle` method. The
+    handler then performs a workflow that is described in the
+    :class:`~.WCSCommonHandler` documentation.
+    
+    This handler follows this workflow with adaptations to the
+    :meth:`createCoverages`, :meth:`configureMapObj` and :meth:`postprocess`
+    methods. The latter one modifies the GetCapabilities response obtained by
+    MapServer to contain EO-WCS specific extensions.
+    """
+    
     SERVICE = "wcs"
     
     REGISTRY_CONF = {
@@ -60,6 +82,11 @@ class WCS20GetCapabilitiesHandler(WCSCommonHandler):
     
     # TODO: override createCoverages, configureRequest, configureMapObj
     def createCoverages(self):
+        """
+        This method adds all Rectified Datasets and Rectified Stitched Mosaics
+        to the ``coverages`` property of the handler. For each of these
+        coverages, a layer will be added to the MapScript :class:`mapObj`.
+        """
         self.coverages = self._get_coverages(
             [
                 "resources.coverages.wrappers.RectifiedDatasetWrapper",
@@ -82,8 +109,35 @@ class WCS20GetCapabilitiesHandler(WCSCommonHandler):
             )
         else:
             return factory.find(filter_exprs=[visible_expr])
+
+
+    def configureMapObj(self):
+        """
+        This method extends the :class:`~.WCSCommonHandler.configureMapObj`
+        method to include informations on the available output formats as
+        well as the supported CRSes.
+        """
+        super(WCS20GetCapabilitiesHandler, self).configureMapObj()
+
+        # set all the supported formats 
+        for output_format in getMSOutputFormatsAll() :  
+            self.map.appendOutputFormat(output_format)
+
+        self.map.setMetaData("wcs_formats",getMSWCSFormatMD()) 
+
+        # set supported CRSes 
+        self.map.setMetaData( 'ows_srs' , getMSWCSSRSMD() ) 
+        self.map.setMetaData( 'wcs_srs' , getMSWCSSRSMD() ) 
+
     
     def getMapServerLayer(self, coverage):
+        """
+        This method returns a MapScript :class:`layerObj` for the input
+        ``coverage``. It extends the
+        :class:`~.WCSCommonHandler.getMapServerLayer` function by configuring
+        the input data using the appropriate connectors (see
+        :mod:`eoxserver.services.connectors`).
+        """
         layer = super(WCS20GetCapabilitiesHandler, self).getMapServerLayer(coverage)
         
         connector = System.getRegistry().findAndBind(
@@ -98,12 +152,46 @@ class WCS20GetCapabilitiesHandler(WCSCommonHandler):
         
         return layer
 
+
     def postprocess(self, resp):
+        """
+        This method transforms the standard WCS 2.0 response ``resp`` obtained
+        from MapServer into an EO-WCS compliant GetCapabilities response and
+        returns the corresponding :class:`~.Response` object.
+        
+        Specifically,
+        
+        * the xsi:schemaLocation attribute of the document root is set
+          to the EO-WCS schema URL
+        * the extensions supported by EOxServer are added to the
+          wcs:ServiceMetadata element
+        * the supported EO-WCS profiles are added to the
+          wcs:ServiceIdentification element
+        * the metadata for the DescribeEOCoverageSet operation is added
+          to the ows:OperationsMetadata element
+        * the wcs:Contents section is replaced by an EO-WCS compliant
+          structure
+          
+        The wcs:Contents section is configured with the coverage summaries of
+        all visible Rectified and Referenceable Datasets, of all Rectified
+        Stitched Mosaics and the summaries of all Dataset Series. Note that
+        the handler is aware of the OWS Common sections parameter which allows
+        to deselect all or parts of the wcs:Contents section and acts
+        accordingly.
+        
+        Should MapServer return an exception report in ``resp``, it is
+        passed on unchanged except for the xsi:schemaLocation attribute.
+        """
+        
         dom = minidom.parseString(resp.content)
         
+        # add the additional CRS namespace 
+
+
         # change xsi:schemaLocation
         schema_location_attr = dom.documentElement.getAttributeNode("xsi:schemaLocation")
         schema_location_attr.nodeValue = "http://www.opengis.net/wcseo/1.0 http://schemas.opengis.net/wcseo/1.0/wcsEOAll.xsd"
+
         
         # we are finished if the response is an ows:ExceptionReport
         # proceed otherwise
@@ -111,9 +199,23 @@ class WCS20GetCapabilitiesHandler(WCSCommonHandler):
         
             encoder = WCS20EOAPEncoder()
 
+            #TODO: Following part should be handled by the MapServer. 
+            #      Remove the code when MapServer really does it. 
+
+            # append SupportedCRSs to ServiceMetadata
+            svc_md = dom.getElementsByTagName("wcs:ServiceMetadata").item(0)
+
+            if svc_md is not None : 
+                extension = encoder.encodeExtension()
+                svc_md.appendChild(extension)
+                supported_crss = encoder.encodeSupportedCRSs() 
+
+                for sc in supported_crss : 
+                    extension.appendChild( sc )
+                
+            # append EO Profiles to ServiceIdentification
             svc_identification = dom.getElementsByTagName("ows:ServiceIdentification").item(0)
             
-            # append EO Profiles to ServiceIdentification
             if svc_identification is not None:
                 eo_profiles = encoder.encodeEOProfiles()
                 
@@ -135,7 +237,8 @@ class WCS20GetCapabilitiesHandler(WCSCommonHandler):
 
                 op_metadata.appendChild(desc_eo_cov_set_op)
                 
-                op_metadata.appendChild(encoder.encodeCountDefaultConstraint(100)) # TODO remove hardcoded number and make it configurable
+                count_default = WCS20ConfigReader().getPagingCountDefault()
+                op_metadata.appendChild(encoder.encodeCountDefaultConstraint(count_default))
 
 
             # rewrite wcs:Contents
@@ -189,3 +292,16 @@ class WCS20GetCapabilitiesHandler(WCSCommonHandler):
         # TODO: integrate WCS Transaction Operation in getcapabilities response
         #return wcst11AlterCapabilities20(resp)
         return resp
+
+
+class WCS20CorrigendumGetCapabilitiesHandler(WCS20GetCapabilitiesHandler):
+
+    REGISTRY_CONF = {
+        "name": "WCS 2.0 GetCapabilities Handler",
+        "impl_id": "services.ows.wcs20.WCS20CorrigendumGetCapabilitiesHandler",
+        "registry_values": {
+            "services.interfaces.service": "wcs",
+            "services.interfaces.version": "2.0.1",
+            "services.interfaces.operation": "getcapabilities"
+        }
+    }
