@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: wcs1x.py 1941 2012-08-08 15:28:01Z martin.paces $
+# $Id: wcs1x.py 2400 2013-03-23 18:40:44Z meissls $
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -28,14 +28,12 @@
 # THE SOFTWARE.
 #-------------------------------------------------------------------------------
 
+import logging
 import os.path
 
 import mapscript
 
-import logging
-
 from eoxserver.core.system import System
-
 from eoxserver.core.util.xmltools import DOMElementToXML
 from eoxserver.core.exceptions import InternalError
 from eoxserver.services.interfaces import (
@@ -43,7 +41,7 @@ from eoxserver.services.interfaces import (
     OperationHandlerInterface
 )
 from eoxserver.services.owscommon import (
-    OWSCommonServiceHandler, OWSCommonVersionHandler
+    OWSCommonServiceHandler, OWSCommonVersionHandler, OWSCommon11ExceptionHandler
 )
 from eoxserver.services.ogc import OGCExceptionHandler
 from eoxserver.services.mapserver import (
@@ -51,13 +49,24 @@ from eoxserver.services.mapserver import (
 )
 from eoxserver.services.exceptions import InvalidRequestException
 from eoxserver.services.ows.wcs.common import (
-    WCSCommonHandler, get_output_format
+    WCSCommonHandler, getMSOutputFormat,
+    getMSOutputFormatsAll, getMSWCSFormatMD,
+    getMSWCSSRSMD, getMSWCSNativeFormat,
+    getMSWCS10NativeFormat, getMSWCS10FormatMD
 )
 
-
 # following import is needed by WCS-T 
 from eoxserver.services.ows.wcst.wcst11AlterCapabilities import wcst11AlterCapabilities11
 
+# format registry 
+from eoxserver.resources.coverages.formats import getFormatRegistry
+
+# crs utilities 
+from eoxserver.resources.coverages import crss
+
+
+#==============================================================================
+# WCS service handler (required by WCS 2.0 as well ) 
 
 class WCSServiceHandler(OWSCommonServiceHandler):
     SERVICE = "wcs"
@@ -72,6 +81,9 @@ class WCSServiceHandler(OWSCommonServiceHandler):
 
 WCSServiceHandlerImplementation = ServiceHandlerInterface.implement(WCSServiceHandler)
 
+#==============================================================================
+# WCS 1.x version handlers
+
 class WCS10VersionHandler(OWSCommonVersionHandler):
     SERVICE = "wcs"
     
@@ -104,6 +116,12 @@ class WCS11VersionHandler(OWSCommonVersionHandler):
             "services.interfaces.version": "1.1.0"
         }
     }
+    
+    def _handleException(self, req, exception):
+        schemas = {
+            "http://www.opengis.net/ows/1.1": "http://schemas.opengis.net/ows/1.1.0/owsExceptionReport.xsd"
+        }
+        return OWSCommon11ExceptionHandler(schemas, "1.1.0").handleException(req, exception)
 
 WCS11VersionHandlerImplementation = VersionHandlerInterface.implement(WCS11VersionHandler)
 
@@ -118,8 +136,17 @@ class WCS112VersionHandler(OWSCommonVersionHandler):
             "services.interfaces.version": "1.1.2"
         }
     }
+    
+    def _handleException(self, req, exception):
+        schemas = {
+            "http://www.opengis.net/ows/1.1": "http://schemas.opengis.net/ows/1.1.0/owsExceptionReport.xsd"
+        }
+        return OWSCommon11ExceptionHandler(schemas, "1.1.2").handleException(req, exception)
 
 WCS112VersionHandlerImplementation = VersionHandlerInterface.implement(WCS112VersionHandler)
+
+#==============================================================================
+# WCS 1.x base operation handler  
     
 class WCS1XOperationHandler(WCSCommonHandler):
     def createCoverages(self):
@@ -129,7 +156,7 @@ class WCS1XOperationHandler(WCSCommonHandler):
         )
         factory = System.getRegistry().bind("resources.coverages.wrappers.EOCoverageFactory")
         
-        # so far, we do not support referenceable grid coverages in WCS 1.X
+        #NOTE: currently, we do not support referenceable grid coverages in WCS 1.X
         
         self.coverages = factory.find(
             impl_ids = (
@@ -139,12 +166,24 @@ class WCS1XOperationHandler(WCSCommonHandler):
             ),
             filter_exprs=[visible_expr]
         )
+
+    def configureMapObj(self):
+        super(WCS1XOperationHandler, self).configureMapObj()
+
+        # set all supported formats 
+        for output_format in getMSOutputFormatsAll():
+            self.map.appendOutputFormat(output_format)
             
     def getMapServerLayer(self, coverage):
         layer = super(WCS1XOperationHandler, self).getMapServerLayer(coverage)
-        
-        layer.setProjection("+init=epsg:%d" % coverage.getSRID())
-        
+
+        # set the detaset projection 
+        layer.setProjection( crss.asProj4Str( coverage.getSRID() ))
+
+        # set per-layer supported CRSes 
+        layer.setMetaData( 'ows_srs' , getMSWCSSRSMD() ) 
+        layer.setMetaData( 'wcs_srs' , getMSWCSSRSMD() ) 
+
         connector = System.getRegistry().findAndBind(
             intf_id = "services.mapserver.MapServerDataConnectorInterface",
             params = {
@@ -154,14 +193,14 @@ class WCS1XOperationHandler(WCSCommonHandler):
         )
         layer = connector.configure(layer, coverage)
         
+        #TODO: Check why the following are set for mosaics only.
         if coverage.getType() == "eo.rect_stitched_mosaic":
             extent = coverage.getExtent()
             size_x, size_y = coverage.getSize()
             
-            layer.setMetaData("wcs_extent", "%.10f %.10f %.10f %.10f" % extent)
-            layer.setMetaData("wcs_resolution", "%.10f %.10f" % ((extent[2]-extent[0]) / float(size_x), (extent[3]-extent[1]) / float(size_y)))
+            layer.setMetaData("wcs_extent", "%.10g %.10g %.10g %.10g" % extent)
+            layer.setMetaData("wcs_resolution", "%.10g %.10g" % ((extent[2]-extent[0]) / float(size_x), (extent[3]-extent[1]) / float(size_y)))
             layer.setMetaData("wcs_size", "%d %d" % (size_x, size_y))
-            layer.setMetaData("wcs_nativeformat", "GTiff")
 
         # set up rangetype metadata information
         rangetype = coverage.getRangeType()
@@ -173,16 +212,19 @@ class WCS1XOperationHandler(WCSCommonHandler):
             layer.setMetaData("wcs_%s_label" % band.name, band.name)
             layer.setMetaData("wcs_%s_interval" % band.name, "%d %d" % rangetype.getAllowedValues())
 
-        layer.setMetaData("wcs_nativeformat", "GTiff") # TODO: make this configurable like in the line above
+        layer.setMetaData( "wcs_imagemode", gdalconst_to_imagemode_string(rangetype.data_type) )
 
-        layer.setMetaData("wcs_formats", "GTiff GTiff_")
-        layer.setMetaData(
-            "wcs_imagemode", 
-            gdalconst_to_imagemode_string(rangetype.data_type)
-        )
+        #set the layer's native format 
+        layer.setMetaData( 'wcs_nativeformat' , getMSWCSNativeFormat(coverage.getData().getSourceFormat()) ) 
+
+        #set the per-layer supported formats 
+        layer.setMetaData( 'wcs_formats' , getMSWCSFormatMD() )
         
         return layer
 
+#==============================================================================
+# WCS 1.x common operation handlers 
+
 class WCS1XDescribeCoverageHandler(WCS1XOperationHandler):
     def createCoverages(self):
         factory = System.getRegistry().bind("resources.coverages.wrappers.EOCoverageFactory")
@@ -201,7 +243,7 @@ class WCS1XDescribeCoverageHandler(WCS1XOperationHandler):
                     " '%s' found."%coverage_id, "NoSuchCoverage", coverage_id)                     
 
             self.coverages.append(coverage)
-        
+
 class WCS1XGetCoverageHandler(WCS1XOperationHandler):
     def createCoverages(self):
         factory = System.getRegistry().bind("resources.coverages.wrappers.EOCoverageFactory")
@@ -228,8 +270,6 @@ class WCS1XGetCoverageHandler(WCS1XOperationHandler):
             super(WCS1XGetCoverageHandler, self)._setParameter(key, value)
             
     def configureMapObj(self):
-        super(WCS1XOperationHandler, self).configureMapObj()
-        
         format_param = self.req.getParamValue("format")
         if not format_param:
             raise InvalidRequestException(
@@ -238,15 +278,12 @@ class WCS1XGetCoverageHandler(WCS1XOperationHandler):
                 "format"
             )
         
-        output_format = get_output_format(
-            format_param,
-            self.coverages[0]
-        )
+        output_format = getMSOutputFormat( format_param, self.coverages[0] )
         
         self.map.appendOutputFormat(output_format)
         self.map.setOutputFormat(output_format)
         
-        logging.debug("WCS20GetCoverageHandler.configureMapObj: %s" % self.map.imagetype)
+        super(WCS1XGetCoverageHandler, self).configureMapObj()
 
 class WCS10GetCapabilitiesHandler(WCS1XOperationHandler):
     REGISTRY_CONF = {
@@ -261,6 +298,9 @@ class WCS10GetCapabilitiesHandler(WCS1XOperationHandler):
     
 WCS10GetCapabilitiesHandlerImplementation = OperationHandlerInterface.implement(WCS10GetCapabilitiesHandler)
 
+#==============================================================================
+# WCS 1.x specific operation handlers 
+
 class WCS11GetCapabilitiesHandler(WCS1XOperationHandler):
     REGISTRY_CONF = {
         "name": "WCS 1.1.0 GetCapabilities Handler",
@@ -323,6 +363,15 @@ class WCS10DescribeCoverageHandler(WCS1XDescribeCoverageHandler):
     PARAM_SCHEMA = {
         "coverageids": {"xml_location": "/{http://www.opengis.net/wcs/1.0.0}Coverage", "xml_type": "string[]", "kvp_key": "coverage", "kvp_type": "stringlist"},
     }
+
+    # special WCS 1.0 format handling  
+    def getMapServerLayer(self, coverage):
+        layer = super(WCS10DescribeCoverageHandler, self).getMapServerLayer(coverage)
+
+        layer.setMetaData( 'wcs_nativeformat' , getMSWCS10NativeFormat(coverage.getData().getSourceFormat()) ) 
+        layer.setMetaData( 'wcs_formats', getMSWCS10FormatMD() ) 
+
+        return layer
     
 WCS10DescribeCoverageHandlerImplementation = OperationHandlerInterface.implement(WCS10DescribeCoverageHandler)
 
