@@ -1,9 +1,10 @@
 #-------------------------------------------------------------------------------
-# $Id: common.py 2135 2012-11-22 17:28:13Z meissls $
+# $Id: common.py 2475 2013-05-14 09:54:45Z meissls $
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
 #          Stephan Meissl <stephan.meissl@eox.at>
+#          Martin Paces <martin.paces@eox.at>
 #
 #-------------------------------------------------------------------------------
 # Copyright (C) 2011 EOX IT Services GmbH
@@ -29,23 +30,41 @@
 
 import logging
 import os
+from uuid import uuid4
 
-from osgeo.gdalconst import GDT_Byte
 import mapscript
 
 from eoxserver.core.system import System
 from eoxserver.core.util.timetools import (
     isotime, getDateTime
 )
-from eoxserver.core.util.geotools import getSRIDFromCRSIdentifier
 from eoxserver.core.exceptions import InternalError, InvalidParameterException
+from eoxserver.contrib import gdal
 from eoxserver.resources.coverages.filters import (
     BoundedArea, TimeInterval
 )
+from eoxserver.resources.coverages.formats import getFormatRegistry
+from eoxserver.resources.coverages import crss
+from eoxserver.resources.coverages.dateline import (
+    extent_crosses_dateline, wrap_extent_around_dateline
+)
+
 from eoxserver.processing.gdal.reftools import create_temporary_vrt
 from eoxserver.services.owscommon import OWSCommonConfigReader
 from eoxserver.services.mapserver import MapServerOperationHandler
 from eoxserver.services.exceptions import InvalidRequestException
+from eoxserver.processing.gdal.vrt import create_simple_vrt
+
+
+logger = logging.getLogger(__name__)
+
+_stripDot = lambda s : s[1:] if s[0] == '.' else s 
+
+def getMSWMSSRSMD(): 
+    """ get the space separated list of CRS EPSG codes to be passed 
+        to MapScript setMedata("wms_srs",...) method 
+        """
+    return " ".join(crss.getSupportedCRS_WMS(format_function=crss.asShortCode)) 
 
 class WMSLayer(object):
     def __init__(self):
@@ -68,6 +87,7 @@ class WMSLayer(object):
         layer.name = self.getName()
         layer.setMetaData("ows_title", self.getName())
         layer.setMetaData("wms_label", self.getName())
+        layer.addProcessing("CLOSE_CONNECTION=CLOSE")
     
         if self.group_name:
             layer.group = self.group_name
@@ -80,7 +100,7 @@ class WMSLayer(object):
             try:
                 os.remove(temp_file)
             except:
-                logging.warning("Could not remove temporary file '%s'" % temp_file)
+                logger.warning("Could not remove temporary file '%s'" % temp_file)
                 
 class WMSEmptyLayer(WMSLayer):
     def __init__(self, layer_name):
@@ -103,13 +123,20 @@ class WMSCoverageLayer(WMSLayer):
         super(WMSCoverageLayer, self).__init__()
         
         self.coverage = coverage
+        self.name = None
         
     def getName(self):
-        return self.coverage.getCoverageId()
+        if self.name:
+            return self.name
+        else:
+            return self.coverage.getCoverageId()
         
     def isRGB(self):
         return self.coverage.getRangeType().name == "RGB"
     
+    def isRGBA(self):
+        return self.coverage.getRangeType().name == "RGBA"
+    
     def isGrayscale(self):
         return self.coverage.getRangeType().name == "Grayscale"
         
@@ -122,9 +149,7 @@ class WMSCoverageLayer(WMSLayer):
     def getBandSelection(self, req):
         bands = self.coverage.getRangeType().bands
         
-        if len(bands) == 1 or len(bands) == 3:
-            return bands
-        elif len(bands) == 2:
+        if len(bands) in range(1, 5): # 1 to 4
             return bands
         else:
             return bands[:3]
@@ -152,13 +177,13 @@ class WMSCoverageLayer(WMSLayer):
             layer.offsite = mapscript.colorObj(*nil_values)
     
     def setScale(self, layer):
-        if self.coverage.getRangeType().data_type != GDT_Byte:
+        if self.coverage.getRangeType().data_type != gdal.GDT_Byte:
             layer.setProcessingKey("SCALE", "AUTO")
     
     def configureBands(self, layer, req):
         bands = self.getBandSelection(req)
         
-        if not self.isRGB() and not self.isGrayscale():
+        if not self.isRGB() and not self.isGrayscale() and not self.isRGBA():
             layer.setProcessingKey("BANDS", "%d,%d,%d" % tuple(self.getBandIndices(req)))
         
         self.setOffsiteColor(layer, bands)
@@ -181,16 +206,21 @@ class WMSCoverageLayer(WMSLayer):
         return layer
 
 class WMSRectifiedDatasetLayer(WMSCoverageLayer):
+    
     def getMapServerLayer(self, req):
         layer = super(WMSRectifiedDatasetLayer, self).getMapServerLayer(req)
 
-        # general rectified coverage stuff
-        layer.setProjection("+init=epsg:%d" % int(self.coverage.getSRID()))
-        layer.setMetaData("ows_srs", "EPSG:%d" % int(self.coverage.getSRID()))
-        layer.setMetaData("wms_extent", "%f %f %f %f" % self.coverage.getExtent())
-        layer.setExtent(*self.coverage.getExtent())
+        # general rectified coverage metadata
+        srid = self.coverage.getSRID()
+        extent = self.coverage.getExtent()
+        
+        layer.setProjection(crss.asProj4Str(srid))
+        layer.setMetaData("ows_srs", crss.asShortCode(srid))
+        layer.setMetaData("wms_srs", crss.asShortCode(srid))
+        layer.setMetaData("wms_extent", "%.10g %.10g %.10g %.10g" % extent)
+        layer.setExtent(*extent)
 
-        # rectified dataset stuff
+        # bind rectified dataset
         connector = System.getRegistry().findAndBind(
             intf_id = "services.mapserver.MapServerDataConnectorInterface",
             params = {
@@ -202,7 +232,51 @@ class WMSRectifiedDatasetLayer(WMSCoverageLayer):
         layer = connector.configure(layer, self.coverage)
 
         return layer
+    
+class WMSWrappedRectifiedDatasetLayer(WMSRectifiedDatasetLayer):
+    
+    def __init__(self, coverage, vrt_path, extent=None):
+        super(WMSWrappedRectifiedDatasetLayer, self).__init__(coverage)
+        self.vrt_path = vrt_path
         
+    
+    def getMapServerLayer(self, req):
+        layer = super(WMSWrappedRectifiedDatasetLayer, self).getMapServerLayer(req)
+
+        e = wrap_extent_around_dateline(self.coverage.getExtent(),
+                                        self.coverage.getSRID())
+        
+        layer.setMetaData("wms_extent", "%.10g %.10g %.10g %.10g" % e)
+        layer.setExtent(*e)
+        
+        data_package = self.coverage.getData()
+        data_package.prepareAccess()
+        ds = gdal.Open(data_package.getGDALDatasetIdentifier())
+        
+        vrt_ds = create_simple_vrt(ds, self.vrt_path)
+        
+        
+        size_x = ds.RasterXSize
+        size_y = ds.RasterYSize
+        
+        dx = abs(e[0] - e[2]) / size_x
+        dy = abs(e[1] - e[3]) / size_y 
+        
+        vrt_ds.SetGeoTransform([e[0], dx, 0, e[3], 0, -dy])
+        
+        vrt_ds = None
+        
+        layer.data = self.vrt_path
+
+        return layer
+    
+    
+    def cleanup(self):
+        super(WMSWrappedRectifiedDatasetLayer, self).cleanup()
+        gdal.Unlink(str(self.vrt_path))
+        
+        
+    
 class WMSReferenceableDatasetLayer(WMSCoverageLayer):
     def setScale(self, layer):
         layer.setProcessingKey("SCALE", "1,2000") # TODO: make this configurable
@@ -210,16 +284,18 @@ class WMSReferenceableDatasetLayer(WMSCoverageLayer):
     def getMapServerLayer(self, req):
         layer = super(WMSReferenceableDatasetLayer, self).getMapServerLayer(req)
 
-        layer.setMetaData("ows_srs", "EPSG:4326")
-        layer.setMetaData("wms_extent", "%f %f %f %f" % self.coverage.getWGS84Extent())
-        layer.setExtent(*self.coverage.getWGS84Extent())
+        # general rectified coverage metadata
+        srid = self.coverage.getSRID()
+        layer.setProjection( crss.asProj4Str( srid ) )
+        layer.setMetaData("ows_srs", crss.asShortCode( srid ) ) 
+        layer.setMetaData("wms_srs", crss.asShortCode( srid ) ) 
+        layer.setMetaData("wms_extent", "%.10g %.10g %.10g %.10g" \
+                                                   % self.coverage.getExtent())
+        layer.setExtent(*self.coverage.getExtent())
         
-        layer.setProjection("+init=epsg:4326") # TODO: read this from dataset or database
-
+        # project the dataset 
         vrt_path = self.rectify()
-        
         layer.data = vrt_path
-        
         self.temp_files.append(vrt_path)
 
         return layer
@@ -233,10 +309,6 @@ class WMSRectifiedStitchedMosaicLayer(WMSCoverageLayer):
     def getMapServerLayer(self, req):
         layer = super(WMSRectifiedStitchedMosaicLayer, self).getMapServerLayer(req)
         
-        layer.setProjection("+init=epsg:%d" % int(self.coverage.getSRID()))
-        layer.setMetaData("ows_srs", "EPSG:%d" % int(self.coverage.getSRID()))
-
-        #
         connector = System.getRegistry().findAndBind(
             intf_id = "services.mapserver.MapServerDataConnectorInterface",
             params = {
@@ -252,16 +324,18 @@ class WMSRectifiedStitchedMosaicLayer(WMSCoverageLayer):
         size = self.coverage.getSize()
         resolution = ((extent[2]-extent[0]) / float(size[0]),
                       (extent[1]-extent[3]) / float(size[1]))
+
+        layer.setProjection( crss.asProj4Str( srid ) )
+        layer.setMetaData("ows_srs", crss.asShortCode( srid ) ) 
+        layer.setMetaData("wms_srs", crss.asShortCode( srid ) ) 
         
         layer.setExtent(*self.coverage.getExtent())
-        layer.setMetaData("wms_extent", "%.10f %.10f %.10f %.10f" % extent)
-        layer.setMetaData("wms_resolution", "%.10f %.10f" % resolution)
+        layer.setMetaData("wms_extent", "%.10g %.10g %.10g %.10g" % extent)
+        layer.setMetaData("wms_resolution", "%.10g %.10g" % resolution)
         layer.setMetaData("wms_size", "%d %d" % size)
         
         layer.type = mapscript.MS_LAYER_RASTER
         layer.setConnectionType(mapscript.MS_RASTER, '')
-        layer.setMetaData("wms_srs", "EPSG:%d" % srid)
-        layer.setProjection("+init=epsg:%d" % srid)
 
         return layer
 
@@ -279,7 +353,8 @@ class WMSDatasetSeriesLayer(WMSLayer):
         
         coverages = self.dataset_series.getEOCoverages()
 
-        layer.setMetaData("wms_extent", "%f %f %f %f" % self.dataset_series.getWGS84Extent())
+        layer.setMetaData("wms_extent", "%.10g %.10g %.10g %.10g" \
+                                        % self.dataset_series.getWGS84Extent())
         layer.setExtent(*self.dataset_series.getWGS84Extent())
         
         time_extent = ",".join(
@@ -287,15 +362,21 @@ class WMSDatasetSeriesLayer(WMSLayer):
         )
         layer.setMetaData("wms_timeextent", time_extent)
         
-        layer.setProjection("+init=epsg:4326")
-        layer.setMetaData("wms_srs", "EPSG:4326")
+        srid = 4326 # TODO: source CRS of dataset series 
+        layer.setProjection( crss.asProj4Str( srid ) )
+        layer.setMetaData("ows_srs", crss.asShortCode( srid ) ) 
+        layer.setMetaData("wms_srs", crss.asShortCode( srid ) ) 
         
+
         layer.type = mapscript.MS_LAYER_RASTER
         
         layer.setConnectionType(mapscript.MS_RASTER, '')
         layer.setMetaData("wms_enable_request", "*")
         layer.status = mapscript.MS_ON
 
+        for key, value in self.dataset_series.getLayerMetadata():
+            layer.setMetaData(key, value)
+
         # use a dummy coverage to connect to
         
         connector = System.getRegistry().findAndBind(
@@ -346,7 +427,9 @@ class WMSCommonHandler(MapServerOperationHandler):
         pass
         
     def _setMapProjection(self):
-        pass
+        # set the default EPSG:4326 projection 
+        # TODO: check whether this is really correct 
+        self.map.setProjection( crss.asProj4Str( 4326 ) ) 
         
     def configureMapObj(self):
         """
@@ -362,12 +445,47 @@ class WMSCommonHandler(MapServerOperationHandler):
         
         self.map.setMetaData("ows_onlineresource", OWSCommonConfigReader().getHTTPServiceURL() + "?")
         
+        # set (per-service) map projection 
         self._setMapProjection()
-    
+
+        # set the (global) list of supported CRSes
+        self.map.setMetaData("ows_srs", getMSWMSSRSMD()) 
+        self.map.setMetaData("wms_srs", getMSWMSSRSMD()) 
+
+        # set all supported output formats 
+
+        # retrieve the format registry 
+        FormatRegistry = getFormatRegistry() 
+
+        # define the supported formats 
+        for sf in FormatRegistry.getSupportedFormatsWMS():
+            # output format definition 
+            of = mapscript.outputFormatObj( sf.driver, "custom" )
+            of.name      = sf.mimeType 
+            of.mimetype  = sf.mimeType 
+            of.extension = _stripDot(sf.defaultExt) 
+            #of.imagemode = mapscript.MS_IMAGEMODE_BYTE
+            of.imagemode = mapscript.MS_IMAGEMODE_RGBA
+
+            #add the format 
+            self.map.appendOutputFormat( of )
+
+        # set the formats supported by getMap WMS operation 
+        self.map.setMetaData("wms_getmap_formatlist", 
+            ",".join(
+                map(lambda f: f.mimeType,
+                    FormatRegistry.getSupportedFormatsWMS())
+            )
+        ) 
+
+
     def createLayers(self):
         pass
         
     def createCoverageLayer(self, coverage):
+        logger.debug("Adding WMS coverage layer for coverage '%s'."
+                     % coverage.getCoverageId())
+        
         if coverage.getType() == "plain":
             raise InternalError(
                 "Plain coverage WMS views are not yet implemented."
@@ -399,8 +517,11 @@ class WMSCommonHandler(MapServerOperationHandler):
             layer.cleanup()
 
 class WMS1XGetCapabilitiesHandler(WMSCommonHandler):
-    def _setMapProjection(self):
-        self.map.setProjection("+init=epsg:4326")
+
+    def configureMapObj(self):
+        super(WMS1XGetCapabilitiesHandler, self).configureMapObj() 
+
+
     
     def createLayers(self):
         visible_expr = System.getRegistry().getFromFactory(
@@ -428,9 +549,9 @@ class WMS1XGetCapabilitiesHandler(WMSCommonHandler):
         return ms_layer
     
 class WMS1XGetMapHandler(WMSCommonHandler):
+
     def _setMapProjection(self):
-        srid = self.getSRID()
-        self.map.setProjection("+init=epsg:%d" % srid)
+        self.map.setProjection( crss.asProj4Str( self.getSRID() ) )
 
     def getSRSParameterName(self):
         raise NotImplementedError()
@@ -556,14 +677,42 @@ class WMS1XGetMapHandler(WMSCommonHandler):
             )
             if coverage is not None:
                 if coverage.matches(filter_exprs):
-                    self.addLayer(self.createCoverageLayer(coverage))
+                    # TODO: check if the coverage crosses the dateline
+                    # if yes, add multiple layers
+                    
+                    if extent_crosses_dateline(coverage.getExtent(), coverage.getSRID()):
+                        logger.debug("Coverage %s crosses the dateline. Special layer setup." % coverage.getCoverageId())
+                        if coverage.getType()  == "eo.rect_dataset":
+                            coverage_id = coverage.getCoverageId()
+                            
+                            unwrapped_coverage_layer = WMSRectifiedDatasetLayer(coverage)
+                            unwrapped_coverage_layer.name = coverage_id + "_unwrapped"
+                            
+                            wrapped_extent = wrap_extent_around_dateline(
+                                coverage.getExtent(), coverage.getSRID()
+                            )
+                            vrt_path = str("/vsimem/%s/%s.vrt" % (str(uuid4()), coverage_id)) # str to avoid unicode issues
+                            wrapped_coverage_layer = WMSWrappedRectifiedDatasetLayer(coverage, vrt_path, wrapped_extent)
+                            wrapped_coverage_layer.name = coverage_id + "_wrapped"
+                            
+                            unwrapped_coverage_layer.setGroup(coverage_id)
+                            wrapped_coverage_layer.setGroup(coverage_id)
+                            
+                            self.addLayer(unwrapped_coverage_layer)
+                            self.addLayer(wrapped_coverage_layer)
+                        else:
+                            raise NotImplementedError(
+                                "WMS for dateline crossing datasets are not "
+                                "implemented."
+                            )
+                    else:
+                        self.addLayer(self.createCoverageLayer(coverage))
                 else:
                     self.addLayer(WMSEmptyLayer(coverage.getCoverageId()))
             else:
                 raise InvalidRequestException(
                     "No coverage or dataset series with EO ID '%s' found" % layer_name,
-                    "LayerNotDefined",
-                    "layers"
+                    "LayerNotDefined", "layers"
                 )
         
     def createDatasetSeriesLayers(self, dataset_series, filter_exprs):
@@ -571,20 +720,47 @@ class WMS1XGetMapHandler(WMSCommonHandler):
             return coverage.getBeginTime()
         
         coverages = dataset_series.getEOCoverages(filter_exprs)
+        eoid = dataset_series.getEOID()
         
         if len(coverages) == 0:
-            layer = WMSEmptyLayer(dataset_series.getEOID())
+            layer = WMSEmptyLayer(eoid)
             
             self.addLayer(layer)
             
         coverages.sort(key=_get_begin_time)
         
         for coverage in coverages:
-            layer = self.createCoverageLayer(coverage)
+            if extent_crosses_dateline(coverage.getExtent(), coverage.getSRID()):
+                logger.debug("Coverage %s crosses the dateline. Special layer setup." % coverage.getCoverageId())
+                if coverage.getType()  == "eo.rect_dataset":
+                    coverage_id = coverage.getCoverageId()
+                    
+                    unwrapped_coverage_layer = WMSRectifiedDatasetLayer(coverage)
+                    unwrapped_coverage_layer.name = coverage_id + "_unwrapped"
+                    
+                    wrapped_extent = wrap_extent_around_dateline(
+                        coverage.getExtent(), coverage.getSRID()
+                    )
+                    vrt_path = str("/vsimem/%s/%s.vrt" % (str(uuid4()), coverage_id)) # str to avoid unicode issues
+                    wrapped_coverage_layer = WMSWrappedRectifiedDatasetLayer(coverage, vrt_path, wrapped_extent)
+                    wrapped_coverage_layer.name = coverage_id + "_wrapped"
+                    
+                    unwrapped_coverage_layer.setGroup(eoid)
+                    wrapped_coverage_layer.setGroup(eoid)
+                    
+                    self.addLayer(unwrapped_coverage_layer)
+                    self.addLayer(wrapped_coverage_layer)
+                else:
+                    raise NotImplementedError(
+                        "WMS for dateline crossing datasets are not "
+                        "implemented."
+                    )
             
-            layer.setGroup(dataset_series.getEOID())
-            
-            self.addLayer(layer)
+            else:    
+                layer = self.createCoverageLayer(coverage)
+                layer.setGroup(eoid)
+                self.addLayer(layer)
+        
             
     def addLayer(self, layer):
         # TODO: more performant solution based on hashes
@@ -595,14 +771,21 @@ class WMS1XGetMapHandler(WMSCommonHandler):
         self.layers.append(layer)
             
     def getSRID(self):
+
         srs = self.req.getParamValue(self.getSRSParameterName())
 
         if srs is None:
-            raise InvalidRequestException("Missing '%s' parameter"% self.getSRSParameterName().upper(), "MissingParameterValue" , self.getSRSParameterName())
+            raise InvalidRequestException("Missing '%s' parameter"
+                % self.getSRSParameterName().upper(), "MissingParameterValue",
+                self.getSRSParameterName())
         
-        srid = getSRIDFromCRSIdentifier(srs)
+        srid = crss.parseEPSGCode(srs,(crss.fromURL,crss.fromURN,
+                    crss.fromShortCode)) 
+
         if srid is None:
-            raise InvalidRequestException("Invalid '%s' parameter value"% self.getSRSParameterName().upper(), "InvalidCRS" , self.getSRSParameterName())
+            raise InvalidRequestException("Invalid '%s' parameter value"
+                % self.getSRSParameterName().upper(), "InvalidCRS" ,
+                self.getSRSParameterName())
             
         return srid
     
