@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# $Id: views.py 2077 2012-10-19 17:13:24Z meissls $
+# $Id: views.py 2123 2012-11-20 15:22:17Z schindlerf $
 #
 # Project: EOxServer <http://eoxserver.org>
 # Authors: Stephan Krause <stephan.krause@eox.at>
@@ -38,6 +38,9 @@ from django.template import RequestContext
 from eoxserver.core.system import System
 from eoxserver import get_version
 
+
+logger = logging.getLogger(__name__)
+
 def index(request):
     System.init()
     dss_factory = System.getRegistry().bind("resources.coverages.wrappers.DatasetSeriesFactory")
@@ -48,10 +51,10 @@ def index(request):
                            impl_ids=["resources.coverages.wrappers.RectifiedStitchedMosaicWrapper"]
                            )]
     
-    # TODO get server URL
+    
     
     return render_to_response(
-        'eoxserver_webclient_index.html', {
+        'webclient/index.html', {
             "datasetseries_eoids": dataset_series_ids,
             "stitchedmosaic_eoids": stitched_mosaic_ids,
             "path": request.path,
@@ -60,7 +63,6 @@ def index(request):
         context_instance=RequestContext(request)
     )
 
-
 def webclient(request, eoid):
     """
     View for webclient interface.
@@ -103,10 +105,10 @@ def webclient(request, eoid):
     ) or "wms"
     
     if preview_service not in ("wms", "wmts"): 
-        logging.error("Unsupported service type '%s'" % preview_service)
+        logger.error("Unsupported service type '%s'" % preview_service)
         preview_service = "wms"
     if outline_service not in ("wms", "wmts"):
-        logging.error("Unsupported service type '%s'" % outline_service)
+        logger.error("Unsupported service type '%s'" % outline_service)
         outline_service = "wms"
     
     preview_url = System.getConfig().getConfigValue(
@@ -118,7 +120,7 @@ def webclient(request, eoid):
     ) or http_ows_url
     
     return render_to_response(
-        'eoxserver_webclient.html', {
+        'webclient/webclient.html', {
             "eoid": eoid,
             "ows_url": http_ows_url,
             "preview_service": preview_service,
@@ -129,8 +131,11 @@ def webclient(request, eoid):
                       "time": begin.strftime("%H:%M")},
             "end": {"date": end.strftime("%Y-%m-%d"),
                     "time": end.strftime("%H:%M")},
-            "extent": "%f,%f,%f,%f" % extent
+            "extent": "%f,%f,%f,%f" % extent,
+            "debug": settings.DEBUG
         },
         context_instance=RequestContext(request)
     )
 
+
+
